function [E,B] = Bloch(alpha,nmax,varargin)
% Solves eigenvector-eigenvalue problem in an optical lattice
% H psi = E psi, where the Hamiltonian H is  
% H = - (1/2)(d/dx)^2 + 2 alpha cos(x)

% Solution is sought as a Bloch function 
% (the product of a plane wave and a periodic function)
% psi(x) = exp(ikx)sum_{-n}^{n} f_{n}exp(1inx)
% Set of equations solved: 
% 0.5*(k + n)^2*f_{n} + alpha (f_{n-1} + f_{n+1}) = E f_{n}

% INPUTS:
% alpha - strength of the lattice
% nmax : harmonics indices n are -nmax <= n <= nmax 
% varargin when present: k - crystal vector (-0.5 < k < 0.5)
% Default k = 0

% OPUTPUTS: 
% E - vector of N eigenenergies E(k)
% B - (N x N) matrix of column eigenvectors (coefficients f_n)

if nargin == 2
    k = 0;
else
    k = varargin{1};
end
N = 2*nmax + 1; 
e = ones(N,1);
kplusm = k + (-nmax:nmax)';
kplusn2 = kplusm.^2;
H = spdiags([alpha*e, 0.5*kplusn2, alpha*e],[-1, 0, 1],N,N);
[B,E] = eig(full(H));
% E - diagonal matrix of eigenvalues
% B - matrix formed by columns of eiegenvectors

%[E,ind] = sort(diag(E)); % extract and sort eigenvalues in ascending order
%B = B(:,ind);
end


